from threading import Thread


class Worker(Thread):

    def __init__(self, queue, func, **thread_kwargs):
        super(Worker, self).__init__(**thread_kwargs)
        self.queue = queue
        self.func = func


class Producer(Worker):

    def run(self):
        while True:
            for task in self.func():
                self.queue.put(task)


class Consumer(Worker):

    def run(self):
        while True:
            task = self.queue.get()
            if task is not None:
                self.func(task)
                self.queue.task_done()



__title__ = "vk-bot"
__version__ = "0.1.5"
__description__ = "Python 2/3 VK bot builder."
__url__ = "https://github.com/Suenweek/vk-bot"
__author__ = "Roman Novatorov"
__author_email__ = "roman.novatorov@gmail.com"



import os
from codecs import open
from setuptools import setup, find_packages


here_dir = os.path.abspath(os.path.dirname(__file__))
about = {}
with open(os.path.join(here_dir, "src", "vk_bot", "__about__.py")) as f:
    exec(f.read(), about)


setup(
    name=about["__title__"],
    version=about["__version__"],
    description=about["__description__"],
    packages=find_packages("src"),
    package_dir={"": "src"},
    url=about["__url__"],
    author=about["__author__"],
    author_email=about["__author_email__"],
    install_requires=[
        "six",
        "vk-client"
    ]
)



from collections import defaultdict
from vk_client import VkClient
from six.moves.queue import Queue
from .workers import Producer, Consumer


class VkBot(object):

    def __init__(self, name, access_token):
        self.name = name
        self.vk = VkClient(access_token)
        self.handlers = defaultdict(list)

    def run(self):
        queue = Queue()
        blp = self.vk.BotsLongPoll.get()

        producer = Producer(queue, func=blp.get_updates)
        consumer = Consumer(queue, func=self.dispatch)

        producer.start()
        consumer.start()

    def add_handler(self, event_type, event_handler):
        self.handlers[event_type].append(event_handler)

    def on(self, event_types):

        def register(event_handler):
            for event_type in event_types:
                self.add_handler(event_type, event_handler)

            return event_handler

        return register

    def dispatch(self, event):
        for event_handler in self.handlers[event.type]:
            event_handler(event.object)
